@@ -1,7 +1,7 @@
 /*
  * file - file I/O routines callable by users
  *
- * Copyright (C) 1999-2007,2014  David I. Bell and Landon Curt Noll
+ * Copyright (C) 1999-2007,2014,2021  David I. Bell and Landon Curt Noll
  *
  * Primary author:  David I. Bell
  *
@@ -41,6 +41,7 @@
 /*
  * Definition of opened files.
  */
+#define MODE_LEN (sizeof("rb+")-1)
 typedef struct {
 	FILEID id;		/* id to identify this file */
 	FILE *fp;		/* real file structure for I/O */
@@ -52,7 +53,7 @@ typedef struct {
 	BOOL appending;		/* TRUE if also opened for appending */
 	BOOL binary;		/* TRUE if binary mode - mode ignored/unused */
 	char action;		/* most recent use for 'r', 'w' or 0 */
-	char mode[sizeof("rb+")+1];/* open mode */
+	char mode[MODE_LEN+1];	/* open mode */
 } FILEIO;
 
 
@@ -63,7 +64,7 @@ typedef struct {
  *	Seek loc bytes from the beginning of the open file, stream.
  *
  * f_tell(FILE *stream, FILEPOS *loc)
- *	Set loc to bytes from the beinning of the open file, stream.
+ *	Set loc to bytes from the beginning of the open file, stream.
  *
  * We assume that if your system does not have fgetpos/fsetpos,
  * then it will have a FILEPOS that is a scalar type (e.g., long).
