@@ -1,7 +1,7 @@
 /*
- * math_error - a simple libcalc math error routine
+ * lib_calc - calc link library initialization and shutdown routines
  *
- * Copyright (C) 1999-2007,2014  Landon Curt Noll
+ * Copyright (C) 1999-2007,2014,2021  Landon Curt Noll
  *
  * Calc is open software; you can redistribute it and/or modify it under
  * the terms of the version 2.1 of the GNU Lesser General Public License
@@ -74,4 +74,15 @@ EXTERN int calc_print_scanwarn_msg;
 /* number of parse/scan warnings found */
 EXTERN unsigned long calc_warn_cnt;
 
+/*
+ * calc history file
+ */
+EXTERN char *calc_history;
+
+/*
+ * calc help
+ */
+EXTERN char *calc_helpdir;
+EXTERN char *calc_customhelpdir;
+
 #endif /* !INCLUDE_MATH_ERROR_H */
