@@ -1,7 +1,7 @@
 /*
  * zmath - declarations for extended precision integer arithmetic
  *
- * Copyright (C) 1999-2002  David I. Bell
+ * Copyright (C) 1999-2007,2014,2021  David I. Bell
  *
  * Calc is open software; you can redistribute it and/or modify it under
  * the terms of the version 2.1 of the GNU Lesser General Public License
@@ -15,11 +15,7 @@
  * A copy of version 2.1 of the GNU Lesser General Public License is
  * distributed with calc under the filename COPYING-LGPL.  You should have
  * received a copy with calc; if not, write to Free Software Foundation, Inc.
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
- *
- * @(#) $Revision: 29.10 $
- * @(#) $Id: zmath.h,v 29.10 2003/01/14 00:44:39 chongo Exp $
- * @(#) $Source: /usr/local/src/cmd/calc/RCS/zmath.h,v $
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  * Under source code control:	1993/07/30 19:42:48
  * File existed as early as:	1993
@@ -30,23 +26,23 @@
 /*
  * Data structure declarations for extended precision integer arithmetic.
  * The assumption made is that a long is 32 bits and shorts are 16 bits,
- * and longs must be addressible on word boundaries.
+ * and longs must be addressable on word boundaries.
  */
 
 
-#if !defined(__ZMATH_H__)
-#define __ZMATH_H__
+#if !defined(INCLUDE_ZMATH_H)
+#define INCLUDE_ZMATH_H
 
 
 #if defined(CALC_SRC)	/* if we are building from the calc source tree */
-# include "calc/win32dll.h"
+# include "calc/decl.h"
 # include "calc/alloc.h"
 # include "calc/endian_calc.h"
 # include "calc/longbits.h"
 # include "calc/byteswap.h"
 # include "calc/have_stdlib.h"
 #else
-# include <calc/win32dll.h>
+# include <calc/decl.h>
 # include <calc/alloc.h>
 # include <calc/endian_calc.h>
 # include <calc/longbits.h>
@@ -125,12 +121,27 @@ typedef SB32 SFULL;			/* signed FULL */
 #define BASE1	(BASE - (FULL)1)		/* one less than base */
 #define BASEDIG ((BASEB/16)*5)			/* number of digits in base */
 #define FULL_BITS (2*BASEB)			/* bits in a FULL */
+#define HALF_LEN (sizeof(HALF))			/* length of HALF in bites */
+#define FULL_LEN (sizeof(FULL))			/* length of FULL in bites */
+
+/*
+ * ROUNDUP(value, mult) - round up value to the next multiple of mult
+ *
+ * NOTE: value and mult musty be of an integer type.
+ *
+ * NOTE: mult must != 0
+ *
+ * NOTE: If value is a multiple of mult, then ROUNDUP(value, mult)
+ *	 will just return value.
+ */
+#define ROUNDUP(value, mult) ( ( ((value)+(mult)-1) / (mult) ) * (mult) )
 
 #define TOPHALF ((FULL)1 << (BASEB-1))		/* highest bit in a HALF */
 #define MAXHALF (TOPHALF - (FULL)1)		/* largest SHALF value */
 
 #define TOPFULL ((FULL)1 << (FULL_BITS-1))	/* highest bit in FULL */
 #define MAXFULL (TOPFULL - (FULL)1)		/* largest SFULL value */
+#define MINSFULL ((SFULL)(TOPFULL))		/* most negative SFULL value */
 #define MAXUFULL (MAXFULL | TOPFULL)		/* largest FULL value */
 
 #define TOPLONG ((unsigned long)1 << (LONG_BITS-1))	/* top long bit */
@@ -143,11 +154,11 @@ typedef SB32 SFULL;			/* signed FULL */
  */
 typedef USB32 QCKHASH;			/* 32 bit hash value */
 #if defined(HAVE_B64) && LONG_BITS == 32
-typedef HALF PRINT;			/* cast for zio printing fucctions */
+typedef HALF PRINT;			/* cast for zio printing functions */
 #define SWAP_B16_IN_PRINT(dest, src)	SWAP_B16_IN_HALF(dest, src)
 #define SWAP_B8_IN_PRINT(dest, src)	SWAP_B8_IN_HALF(dest, src)
 #else
-typedef FULL PRINT;			/* cast for zio printing fucctions */
+typedef FULL PRINT;			/* cast for zio printing functions */
 #define SWAP_B16_IN_PRINT(dest, src)	SWAP_B16_IN_FULL(dest, src)
 #define SWAP_B8_IN_PRINT(dest, src)	SWAP_B8_IN_FULL(dest, src)
 #endif
@@ -179,12 +190,12 @@ typedef SB32 LEN;			/* unit of length storage */
 
 
 /*
- * FNV-1 basis
+ * Quickhash basis
  *
  * We start the hash at a non-zero value at the beginning so that
  * hashing blocks of data with all 0 bits do not map onto the same
- * 0 hash value.  The virgin value that we use below is the hash value
- * that we would get from following 32 ASCII characters:
+ * 0 hash value.  The virgin value that we use below is the 32-bit
+ * FNV-0 hash value that we would get from following 32 ASCII characters:
  *
  *		chongo <Landon Curt Noll> /\../\
  *
@@ -212,7 +223,7 @@ typedef SB32 LEN;			/* unit of length storage */
  * to be a cryptographic hash function, just a fast and reasonably
  * good hash function.
  */
-#define FNV1_32_BASIS ((QCKHASH)(0x811c9dc5))
+#define QUICKHASH_BASIS ((QCKHASH)(0x811c9dc5))
 
 
 /*
@@ -228,11 +239,12 @@ typedef SB32 LEN;			/* unit of length storage */
 #define MAXLEN	((LEN) 0x7fffffff >> 3) /* longest value allowed */
 
 
-#define MAXREDC 5			/* number of entries in REDC cache */
-#define SQ_ALG2 20			/* size for alternative squaring */
-#define MUL_ALG2 20			/* size for alternative multiply */
-#define POW_ALG2 40			/* size for using REDC for powers */
-#define REDC_ALG2 50			/* size for using alternative REDC */
+#define MAXREDC 256			/* number of entries in REDC cache */
+#define SQ_ALG2 28			/* size for alternative squaring */
+#define MUL_ALG2 28			/* size for alternative multiply */
+#define POW_ALG2 20			/* size for using REDC for powers */
+/* old REDC_ALG2 was 5/4 of POW_ALG2, so we will keep the same ratio */
+#define REDC_ALG2 25			/* size for using alternative REDC */
 
 
 typedef union {
@@ -276,126 +288,130 @@ typedef struct {
 /*
  * Function prototypes for integer math routines.
  */
-extern DLL HALF * alloc(LEN len);
+E_FUNC HALF * alloc(LEN len);
 #ifdef	ALLOCTEST
-extern DLL void freeh(HALF *);
+E_FUNC void freeh(HALF *);
 #endif
 
 
 /*
  * Input, output, and conversion routines.
  */
-extern DLL void zcopy(ZVALUE z, ZVALUE *res);
-extern DLL void itoz(long i, ZVALUE *res);
-extern DLL void utoz(FULL i, ZVALUE *res);
-extern DLL void str2z(char *s, ZVALUE *res);
-extern DLL long ztoi(ZVALUE z);
-extern DLL FULL ztou(ZVALUE z);
-extern DLL void zprintval(ZVALUE z, long decimals, long width);
-extern DLL void zprintx(ZVALUE z, long width);
-extern DLL void zprintb(ZVALUE z, long width);
-extern DLL void zprinto(ZVALUE z, long width);
-extern DLL void fitzprint(ZVALUE, long, long);
+E_FUNC void zcopy(ZVALUE z, ZVALUE *res);
+E_FUNC void itoz(long i, ZVALUE *res);
+E_FUNC void utoz(FULL i, ZVALUE *res);
+E_FUNC void stoz(SFULL i, ZVALUE *res);
+E_FUNC void str2z(char *s, ZVALUE *res);
+E_FUNC long ztoi(ZVALUE z);
+E_FUNC FULL ztou(ZVALUE z);
+E_FUNC SFULL ztos(ZVALUE z);
+E_FUNC void zprintval(ZVALUE z, long decimals, long width);
+E_FUNC void zprintx(ZVALUE z, long width);
+E_FUNC void zprintb(ZVALUE z, long width);
+E_FUNC void zprinto(ZVALUE z, long width);
+E_FUNC void fitzprint(ZVALUE, long, long);
 
 
 /*
  * Basic numeric routines.
  */
-extern DLL void zmuli(ZVALUE z, long n, ZVALUE *res);
-extern DLL long zdivi(ZVALUE z, long n, ZVALUE *res);
-extern DLL long zmodi(ZVALUE z, long n);
-extern DLL void zadd(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL void zsub(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL void zmul(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL long zdiv(ZVALUE z1, ZVALUE z2, ZVALUE *res, ZVALUE *rem, long R);
-extern DLL long zquo(ZVALUE z1, ZVALUE z2, ZVALUE *res, long R);
-extern DLL long zmod(ZVALUE z1, ZVALUE z2, ZVALUE *rem, long R);
-extern DLL void zequo(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL BOOL zdivides(ZVALUE z1, ZVALUE z2);
-extern DLL void zor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL void zand(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL void zxor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL void zandnot(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL long zpopcnt(ZVALUE z, int bitval);
-extern DLL void zshift(ZVALUE z, long n, ZVALUE *res);
-extern DLL void zsquare(ZVALUE z, ZVALUE *res);
-extern DLL long zlowbit(ZVALUE z);
-extern DLL LEN zhighbit(ZVALUE z);
-extern DLL void zbitvalue(long n, ZVALUE *res);
-extern DLL BOOL zisset(ZVALUE z, long n);
-extern DLL BOOL zisonebit(ZVALUE z);
-extern DLL BOOL zisallbits(ZVALUE z);
-extern DLL FLAG ztest(ZVALUE z);
-extern DLL FLAG zrel(ZVALUE z1, ZVALUE z2);
-extern DLL FLAG zabsrel(ZVALUE z1, ZVALUE z2);
-extern DLL BOOL zcmp(ZVALUE z1, ZVALUE z2);
+E_FUNC void zmuli(ZVALUE z, long n, ZVALUE *res);
+E_FUNC long zdivi(ZVALUE z, long n, ZVALUE *res);
+E_FUNC long zmodi(ZVALUE z, long n);
+E_FUNC void zadd(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC void zsub(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC void zmul(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC long zdiv(ZVALUE z1, ZVALUE z2, ZVALUE *res, ZVALUE *rem, long R);
+E_FUNC long zquo(ZVALUE z1, ZVALUE z2, ZVALUE *res, long R);
+E_FUNC long zmod(ZVALUE z1, ZVALUE z2, ZVALUE *rem, long R);
+E_FUNC void zequo(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC BOOL zdivides(ZVALUE z1, ZVALUE z2);
+E_FUNC void zor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC void zand(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC void zxor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC void zandnot(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC long zpopcnt(ZVALUE z, int bitval);
+E_FUNC void zshift(ZVALUE z, long n, ZVALUE *res);
+E_FUNC void zsquare(ZVALUE z, ZVALUE *res);
+E_FUNC long zlowbit(ZVALUE z);
+E_FUNC LEN zhighbit(ZVALUE z);
+E_FUNC void zbitvalue(long n, ZVALUE *res);
+E_FUNC BOOL zisset(ZVALUE z, long n);
+E_FUNC BOOL zisonebit(ZVALUE z);
+E_FUNC BOOL zisallbits(ZVALUE z);
+E_FUNC FLAG ztest(ZVALUE z);
+E_FUNC FLAG zrel(ZVALUE z1, ZVALUE z2);
+E_FUNC FLAG zabsrel(ZVALUE z1, ZVALUE z2);
+E_FUNC BOOL zcmp(ZVALUE z1, ZVALUE z2);
 
 
 /*
  * More complicated numeric functions.
  */
-extern DLL FULL uugcd(FULL i1, FULL i2);
-extern DLL long iigcd(long i1, long i2);
-extern DLL void zgcd(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL void zlcm(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL void zreduce(ZVALUE z1, ZVALUE z2, ZVALUE *z1res, ZVALUE *z2res);
-extern DLL void zfact(ZVALUE z, ZVALUE *dest);
-extern DLL void zperm(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL int zcomb(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL FLAG zjacobi(ZVALUE z1, ZVALUE z2);
-extern DLL void zfib(ZVALUE z, ZVALUE *res);
-extern DLL void zpowi(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL void ztenpow(long power, ZVALUE *res);
-extern DLL void zpowermod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res);
-extern DLL BOOL zmodinv(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL BOOL zrelprime(ZVALUE z1, ZVALUE z2);
-extern DLL long zlog(ZVALUE z1, ZVALUE z2);
-extern DLL long zlog10(ZVALUE z);
-extern DLL long zdivcount(ZVALUE z1, ZVALUE z2);
-extern DLL long zfacrem(ZVALUE z1, ZVALUE z2, ZVALUE *rem);
-extern DLL long zgcdrem(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL long zdigits(ZVALUE z1);
-extern DLL long zdigit(ZVALUE z1, long n);
-extern DLL FLAG zsqrt(ZVALUE z1, ZVALUE *dest, long R);
-extern DLL void zroot(ZVALUE z1, ZVALUE z2, ZVALUE *dest);
-extern DLL BOOL zissquare(ZVALUE z);
-extern DLL void zhnrmod(ZVALUE v, ZVALUE h, ZVALUE zn, ZVALUE zr, ZVALUE *res);
+E_FUNC FULL uugcd(FULL i1, FULL i2);
+E_FUNC long iigcd(long i1, long i2);
+E_FUNC void zgcd(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC void zlcm(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC void zreduce(ZVALUE z1, ZVALUE z2, ZVALUE *z1res, ZVALUE *z2res);
+E_FUNC void zfact(ZVALUE z, ZVALUE *dest);
+E_FUNC void zperm(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC int zcomb(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC FLAG zjacobi(ZVALUE z1, ZVALUE z2);
+E_FUNC void zfib(ZVALUE z, ZVALUE *res);
+E_FUNC void zpowi(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC void ztenpow(long power, ZVALUE *res);
+E_FUNC void zpowermod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res);
+E_FUNC BOOL zmodinv(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC BOOL zrelprime(ZVALUE z1, ZVALUE z2);
+E_FUNC long zlog(ZVALUE z1, ZVALUE z2);
+E_FUNC long zlog10(ZVALUE z, BOOL *was_10_power);
+E_FUNC long zdivcount(ZVALUE z1, ZVALUE z2);
+E_FUNC long zfacrem(ZVALUE z1, ZVALUE z2, ZVALUE *rem);
+E_FUNC long zgcdrem(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC long zdigits(ZVALUE z1);
+E_FUNC long zdigit(ZVALUE z1, long n);
+E_FUNC FLAG zsqrt(ZVALUE z1, ZVALUE *dest, long R);
+E_FUNC void zroot(ZVALUE z1, ZVALUE z2, ZVALUE *dest);
+E_FUNC BOOL zissquare(ZVALUE z);
+E_FUNC void zhnrmod(ZVALUE v, ZVALUE h, ZVALUE zn, ZVALUE zr, ZVALUE *res);
 
 
 /*
  * Prime related functions.
  */
-extern DLL FLAG zisprime(ZVALUE z);
-extern DLL FULL znprime(ZVALUE z);
-extern DLL FULL next_prime(FULL v);
-extern DLL FULL zpprime(ZVALUE z);
-extern DLL void zpfact(ZVALUE z, ZVALUE *dest);
-extern DLL BOOL zprimetest(ZVALUE z, long count, ZVALUE skip);
-extern DLL BOOL zredcprimetest(ZVALUE z, long count, ZVALUE skip);
-extern DLL BOOL znextcand(ZVALUE z1, long count, ZVALUE skip, ZVALUE res, ZVALUE mod, ZVALUE *cand);
-extern DLL BOOL zprevcand(ZVALUE z1, long count, ZVALUE skip, ZVALUE res, ZVALUE mod, ZVALUE *cand);
-extern DLL FULL zlowfactor(ZVALUE z, long count);
-extern DLL FLAG zfactor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL long zpix(ZVALUE z1);
-extern DLL void zlcmfact(ZVALUE z, ZVALUE *dest);
+E_FUNC FLAG zisprime(ZVALUE z);
+E_FUNC FULL znprime(ZVALUE z);
+E_FUNC FULL next_prime(FULL v);
+E_FUNC FULL zpprime(ZVALUE z);
+E_FUNC void zpfact(ZVALUE z, ZVALUE *dest);
+E_FUNC BOOL zprimetest(ZVALUE z, long count, ZVALUE skip);
+E_FUNC BOOL zredcprimetest(ZVALUE z, long count, ZVALUE skip);
+E_FUNC BOOL znextcand(ZVALUE z1, long count, ZVALUE skip, ZVALUE res,
+		      ZVALUE mod, ZVALUE *cand);
+E_FUNC BOOL zprevcand(ZVALUE z1, long count, ZVALUE skip, ZVALUE res,
+		      ZVALUE mod, ZVALUE *cand);
+E_FUNC FULL zlowfactor(ZVALUE z, long count);
+E_FUNC FLAG zfactor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC long zpix(ZVALUE z1);
+E_FUNC void zlcmfact(ZVALUE z, ZVALUE *dest);
 
 
 /*
  * Misc misc functions. :-)
  */
-extern DLL void zsquaremod(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL void zminmod(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL BOOL zcmpmod(ZVALUE z1, ZVALUE z2, ZVALUE z3);
-extern DLL void zio_init(void);
+E_FUNC void zsquaremod(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC void zminmod(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC BOOL zcmpmod(ZVALUE z1, ZVALUE z2, ZVALUE z3);
+E_FUNC void zio_init(void);
 
 
 /*
  * These functions are for internal use only.
  */
-extern DLL void ztrim(ZVALUE *z);
-extern DLL void zshiftr(ZVALUE z, long n);
-extern DLL void zshiftl(ZVALUE z, long n);
-extern DLL HALF *zalloctemp(LEN len);
+E_FUNC void ztrim(ZVALUE *z);
+E_FUNC void zshiftr(ZVALUE z, long n);
+E_FUNC void zshiftl(ZVALUE z, long n);
+E_FUNC HALF *zalloctemp(LEN len);
 
 
 /*
@@ -412,26 +428,27 @@ typedef struct {
 	ZVALUE one;		/* REDC format for the number 1 */
 } REDC;
 
-extern DLL REDC *zredcalloc(ZVALUE z1);
-extern DLL void zredcfree(REDC *rp);
-extern DLL void zredcencode(REDC *rp, ZVALUE z1, ZVALUE *res);
-extern DLL void zredcdecode(REDC *rp, ZVALUE z1, ZVALUE *res);
-extern DLL void zredcmul(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern DLL void zredcsquare(REDC *rp, ZVALUE z1, ZVALUE *res);
-extern DLL void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC REDC *zredcalloc(ZVALUE z1);
+E_FUNC void zredcfree(REDC *rp);
+E_FUNC void zredcencode(REDC *rp, ZVALUE z1, ZVALUE *res);
+E_FUNC void zredcdecode(REDC *rp, ZVALUE z1, ZVALUE *res);
+E_FUNC void zredcmul(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC void zredcsquare(REDC *rp, ZVALUE z1, ZVALUE *res);
+E_FUNC void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 
 
 /*
  * macro expansions to speed this thing up
  */
-#define ziseven(z)	(!(*(z).v & 01))
-#define zisodd(z)	(*(z).v & 01)
+#define ziseven(z)	(!(*(z).v & 0x1))
+#define zisodd(z)	(*(z).v & 0x1)
 #define ziszero(z)	((*(z).v == 0) && ((z).len == 1))
 #define zisneg(z)	((z).sign)
 #define zispos(z)	(((z).sign == 0) && (*(z).v || ((z).len > 1)))
 #define zisunit(z)	((*(z).v == 1) && ((z).len == 1))
 #define zisone(z)	((*(z).v == 1) && ((z).len == 1) && !(z).sign)
 #define zisnegone(z)	((*(z).v == 1) && ((z).len == 1) && (z).sign)
+#define zltnegone(z)	(zisneg(z) && (((z).len > 1) || *(z).v > 1))
 #define zistwo(z)	((*(z).v == 2) && ((z).len == 1) && !(z).sign)
 #define zisabstwo(z)	((*(z).v == 2) && ((z).len == 1))
 #define zisabsleone(z)	((*(z).v <= 1) && ((z).len == 1))
@@ -442,7 +459,8 @@ extern DLL void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 /*
  * zgtmaxfull(z)	TRUE if abs(z) > MAXFULL
  */
-#define zgtmaxfull(z)	(((z).len > 2) || (((z).len == 2) && (((SHALF)(z).v[1]) < 0)))
+#define zgtmaxfull(z)	(((z).len > 2) || (((z).len == 2) && \
+			 (((SHALF)(z).v[1]) < 0)))
 
 /*
  * zgtmaxufull(z)	TRUE if abs(z) will not fit into a FULL (> MAXUFULL)
@@ -462,14 +480,15 @@ extern DLL void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
  * zgtmaxlong(z)	TRUE if abs(z) > MAXLONG
  */
 #if BASEB >= LONG_BITS
-#define zgtmaxlong(z)	(((z).len > 1) || (((z).len == 1) && (((SHALF)(z).v[0]) < 0)))
+#define zgtmaxlong(z)	(((z).len > 1) || (((z).len == 1) && \
+			 (((SHALF)(z).v[0]) < 0)))
 #else
 #define zgtmaxlong(z)	zgtmaxfull(z)
 #endif
 
 /*
  * Some algorithms testing for values of a certain length.  Macros such as
- * zistiny() do this well.  In other cases algorthms require tests for values
+ * zistiny() do this well.  In other cases algorithms require tests for values
  * in comparison to a given power of 2.	 In the later case, zistiny() compares
  * against a different power of 2 on a 64 bit machine.	The macros below
  * provide a tests against powers of 2 that are independent of the work size.
@@ -479,6 +498,13 @@ extern DLL void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
  *	zge31b(z)	TRUE if abs(z) >= 2^31
  *	zge32b(z)	TRUE if abs(z) >= 2^32
  *	zge64b(z)	TRUE if abs(z) >= 2^64
+ *	zge128b(z)	TRUE if abs(z) >= 2^128
+ *	zge256b(z)	TRUE if abs(z) >= 2^256
+ *	zge512b(z)	TRUE if abs(z) >= 2^512
+ *	zge1024b(z)	TRUE if abs(z) >= 2^1024
+ *	zge2048b(z)	TRUE if abs(z) >= 2^2048
+ *	zge4096b(z)	TRUE if abs(z) >= 2^4096
+ *	zge8192b(z)	TRUE if abs(z) >= 2^8192
  */
 #if BASEB == 32
 
@@ -487,14 +513,30 @@ extern DLL void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 #define zge31b(z)	(!zistiny(z) || (((SHALF)(z).v[0]) < 0))
 #define zge32b(z)	(!zistiny(z))
 #define zge64b(z)	((z).len > 2)
+#define zge128b(z)	((z).len > 4)
+#define zge256b(z)	((z).len > 8)
+#define zge512b(z)	((z).len > 16)
+#define zge1024b(z)	((z).len > 32)
+#define zge2048b(z)	((z).len > 64)
+#define zge4096b(z)	((z).len > 128)
+#define zge8192b(z)	((z).len > 256)
 
 #else
 
 #define zge16b(z)	(!zistiny(z))
-#define zge24b(z)	(((z).len > 2) || (((z).len == 2) && ((z).v[1] >= (HALF)0x100)))
-#define zge31b(z)	(((z).len > 2) || (((z).len == 2) && (((SHALF)(z).v[1]) < 0)))
+#define zge24b(z)	(((z).len > 2) || (((z).len == 2) && \
+			 ((z).v[1] >= (HALF)0x100)))
+#define zge31b(z)	(((z).len > 2) || (((z).len == 2) && \
+			 (((SHALF)(z).v[1]) < 0)))
 #define zge32b(z)	((z).len > 2)
 #define zge64b(z)	((z).len > 4)
+#define zge128b(z)	((z).len > 8)
+#define zge256b(z)	((z).len > 16)
+#define zge512b(z)	((z).len > 32)
+#define zge1024b(z)	((z).len > 64)
+#define zge2048b(z)	((z).len > 128)
+#define zge4096b(z)	((z).len > 256)
+#define zge8192b(z)	((z).len > 512)
 
 #endif
 
@@ -551,73 +593,80 @@ extern DLL void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 #define MODE_HEX	5
 #define MODE_OCTAL	6
 #define MODE_BINARY	7
-#define MODE_MAX	7
+#define MODE_REAL_AUTO	8
+#define MODE_ENG    9
+#define MODE_MAX	9
 #define MODE2_OFF	(MODE_MAX+1)
 
+/* XXX - perhaps we need the MODE_REAL_AUTO vs MODE_REAL as a config mode? */
+#if 0 	/* XXX - can we safely set MODE_INITIAL to MODE_REAL_AUTO ?? */
+#define MODE_INITIAL	MODE_REAL_AUTO
+#else
 #define MODE_INITIAL	MODE_REAL
+#endif
 #define MODE2_INITIAL	MODE2_OFF
 
 
 /*
  * Output routines for either FILE handles or strings.
  */
-extern DLL void math_chr(int ch);
-extern DLL void math_str(char *str);
-extern DLL void math_fill(char *str, long width);
-extern DLL void math_flush(void);
-extern DLL void math_divertio(void);
-extern DLL void math_cleardiversions(void);
-extern DLL char *math_getdivertedio(void);
-extern DLL int math_setmode(int mode);
-extern DLL int math_setmode2(int mode);
-extern DLL LEN math_setdigits(LEN digits);
-extern DLL void math_fmt(char *, ...);
+E_FUNC void math_chr(int ch);
+E_FUNC void math_str(char *str);
+E_FUNC void math_fill(char *str, long width);
+E_FUNC void math_flush(void);
+E_FUNC void math_divertio(void);
+E_FUNC void math_cleardiversions(void);
+E_FUNC char *math_getdivertedio(void);
+E_FUNC int math_setmode(int mode);
+E_FUNC int math_setmode2(int mode);
+E_FUNC LEN math_setdigits(LEN digits);
+E_FUNC void math_fmt(char *, ...) PRINTF_FORMAT(1, 2);
 
 
 /*
  * The error routine.
  */
-extern DLL void math_error(char *, ...);
+E_FUNC void math_error(char *, ...) PRINTF_FORMAT(1, 2);
 
 
 /*
  * external swap functions
  */
-extern DLL HALF *swap_b8_in_HALFs(HALF *dest, HALF *src, LEN len);
-extern DLL ZVALUE *swap_b8_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
-extern DLL HALF *swap_b16_in_HALFs(HALF *dest, HALF *src, LEN len);
-extern DLL ZVALUE *swap_b16_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
-extern DLL ZVALUE *swap_HALF_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
+E_FUNC HALF *swap_b8_in_HALFs(HALF *dest, HALF *src, LEN len);
+E_FUNC ZVALUE *swap_b8_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
+E_FUNC HALF *swap_b16_in_HALFs(HALF *dest, HALF *src, LEN len);
+E_FUNC ZVALUE *swap_b16_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
+E_FUNC ZVALUE *swap_HALF_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
 
 
 /*
  * constants used often by the arithmetic routines
  */
-extern HALF _zeroval_[], _oneval_[], _twoval_[], _threeval_[], _fourval_[];
-extern HALF _fiveval_[], _sixval_[], _sevenval_[], _eightval_[], _nineval_[];
-extern HALF _tenval_[], _elevenval_[], _twelveval_[], _thirteenval_[];
-extern HALF _fourteenval_[], _fifteenval_[];
-extern HALF _sqbaseval_[];
-extern HALF _fourthbaseval_[];
+EXTERN HALF _zeroval_[], _oneval_[], _twoval_[], _threeval_[], _fourval_[];
+EXTERN HALF _fiveval_[], _sixval_[], _sevenval_[], _eightval_[], _nineval_[];
+EXTERN HALF _tenval_[], _elevenval_[], _twelveval_[], _thirteenval_[];
+EXTERN HALF _fourteenval_[], _fifteenval_[];
+EXTERN HALF _sqbaseval_[];
+EXTERN HALF _fourthbaseval_[];
 
-extern ZVALUE zconst[];		/* ZVALUE integers from 0 thru 15 */
+EXTERN ZVALUE zconst[];		/* ZVALUE integers from 0 thru 15 */
 
-extern ZVALUE _zero_, _one_, _two_, _ten_, _neg_one_;
-extern ZVALUE _sqbase_, _pow4base_, _pow8base_;
+EXTERN ZVALUE _zero_, _one_, _two_, _ten_, _neg_one_;
+EXTERN ZVALUE _sqbase_, _pow4base_, _pow8base_;
 
-extern ZVALUE _b32_, _b64_;
+EXTERN ZVALUE _b32_, _b64_;
 
-extern DLL BOOL _math_abort_;	/* nonzero to abort calculations */
-extern ZVALUE _tenpowers_[];	/* table of 10^2^n */
+EXTERN BOOL _math_abort_;	/* nonzero to abort calculations */
+EXTERN ZVALUE _tenpowers_[];	/* table of 10^2^n */
 
 /*
- * Bit fiddeling functions and types
+ * Bit fiddling functions and types
  */
-extern HALF bitmask[];		/* bit rotation, norm 0 */
-extern HALF lowhalf[];		/* bit masks from low end of HALF */
-extern HALF rlowhalf[];		/* reversed bit masks from low end of HALF */
-extern HALF highhalf[];		/* bit masks from high end of HALF */
-extern HALF rhighhalf[];	/* reversed bit masks from high end of HALF */
+EXTERN HALF bitmask[];		/* bit rotation, norm 0 */
+EXTERN HALF lowhalf[];		/* bit masks from low end of HALF */
+EXTERN HALF rlowhalf[];		/* reversed bit masks from low end of HALF */
+EXTERN HALF highhalf[];		/* bit masks from high end of HALF */
+EXTERN HALF rhighhalf[];	/* reversed bit masks from high end of HALF */
 #define HAVE_REVERSED_MASKS	/* allows old code to know libcalc.a has them */
 
 
@@ -637,35 +686,4 @@ typedef struct {
 } BITSTR;
 
 
-/*
- * HVAL(a,b) - form an array of HALFs given 8 hex digits
- *		  a: up to 4 hex digits without the leading 0x (upper half)
- *		  b: up to 4 hex digits without the leading 0x (lower half)
- *
- *	NOTE: Due to a SunOS cc bug, don't put spaces in the HVAL call!
- */
-#if FULL_BITS == 64
-
-# if defined(FORCE_STDC) || (defined(__STDC__) && __STDC__ != 0) || defined(__cplusplus)
-#  define HVAL(a,b) (HALF)(0x ## a ## b)
-# else
-#  define HVAL(a,b) (HALF)(0x/**/a/**/b)
-# endif
-
-#elif 2*FULL_BITS == 64
-
-# if defined(FORCE_STDC) || (defined(__STDC__) && __STDC__ != 0) || defined(__cplusplus)
-#  define HVAL(a,b) (HALF)0x##b, (HALF)0x##a
-# else
-   /* NOTE: Due to a SunOS cc bug, don't put spaces in the HVAL call! */
-#  define HVAL(a,b) (HALF)0x/**/b, (HALF)0x/**/a
-# endif
-
-#else
-
-   /\../\	FULL_BITS must be 32 or 64	/\../\	 !!!
-
-#endif
-
-
-#endif /* !__ZMATH_H__*/
+#endif /* !INCLUDE_ZMATH_H*/
